//Copyright © 2015, Oracle and/or its affiliates. All rights reserved.

package server;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.URI;
import java.util.Objects;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.glassfish.jersey.jdkhttp.JdkHttpServerFactory;
import org.glassfish.jersey.server.ResourceConfig;

import resource.DepartmentResource;
import resource.EmployeeResource;
import resource.SimpleResource;

import com.owlike.genson.ext.jaxrs.GensonJsonConverter;
import com.sun.net.httpserver.HttpServer;



public final class Server {
	public static Server start(URI uri) throws Exception {
		Server server = new Server(uri);
		server.start();
		return server;
	}

	private URI uri;
	private HttpServer httpServer;

	private Server(URI uri) {
		this.uri = Objects.requireNonNull(uri);
	}

	public URI getURI() {
		return uri;
	}

	private void start() throws Exception {
		ResourceConfig resourceConfig = createResourceConfig();
		httpServer = JdkHttpServerFactory.createHttpServer(getURI(), resourceConfig, false);
		httpServer.setExecutor(newFixedCachedThreadPool(10, new NamedThreadFactory("OEPE Test Server")));
		httpServer.start();
	}

	protected ResourceConfig createResourceConfig() {
		return new ResourceConfig(
				SimpleResource.class,
				EmployeeResource.class,
				DepartmentResource.class
		).register(GensonJsonConverter.class);
	}

	public synchronized void stop() throws Exception {
		if (httpServer != null) {
			httpServer.stop(0);
			httpServer = null;
			Thread.sleep(2);
		}
	}
	
	public static ExecutorService newFixedCachedThreadPool(int nThreads, ThreadFactory threadFactory) {
		return new ThreadPoolExecutor(
			nThreads/2,
			nThreads,
			60, TimeUnit.SECONDS,
			new ArrayBlockingQueue<Runnable>(nThreads),
			Objects.requireNonNull(threadFactory),
			new ThreadPoolExecutor.CallerRunsPolicy());
	}

	public static void main(String[] args) throws Exception {
		System.out.print("Enter port number [4545]: ");
		BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));
		String port = reader.readLine();
		if (port == null || "".equals(port)) {
			port = "4545";
		}

		URI uri = new URI("http://localhost:" + port + '/');
		Server server = Server.start(uri);

		System.out.println("Server started at " + uri);
		System.out.println("Press <enter> to exit...");
		reader.readLine();

		server.stop();

		System.out.println("Done");
		System.exit(0);
	}
}